@@ -8,10 +8,7 @@ logger = logging.getLogger(__name__)
 
 
 class _Symmetry(object):
-    """A class to handle point and space group symmetry.
-
-    :meta public:
-    """
+    """A class to handle point and space group symmetry."""
 
     spgno_to_hall = None
     spgname_to_hall = None
