@@ -4,6 +4,7 @@
 """Tests for `cell` in the `molsystem` package."""
 
 import pytest  # noqa: F401
+import numpy as np
 
 from molsystem import Cell
 
@@ -52,6 +53,25 @@ def test_cell_object(vanadium):
     assert cell == [3.03, 3.03, 3.03, 90.0, 90.0, 90.0]
 
 
+def test_cell_strain_yy(vanadium):
+    """Test straining the cell in the yy direction."""
+    cell = vanadium.cell
+    assert str(type(cell)) == "<class 'molsystem.cell._Cell'>"
+    cell.strain(0, 0.01, 0, 0, 0, 0)
+    assert cell == [3.03, 3.0603, 3.03, 90.0, 90.0, 90.0]
+
+
+def test_cell_strain_xz(vanadium):
+    """Test getting the cell as a Cell object."""
+    cell = vanadium.cell
+    assert str(type(cell)) == "<class 'molsystem.cell._Cell'>"
+    cell.strain([0, 0, 0, 0, 0.01, 0])
+    answer = [3.030037874763284, 3.03, 3.030037874763284, 90.0, 89.42704697944585, 90.0]
+    if not np.allclose(cell.parameters, answer):
+        print(cell)
+    assert np.allclose(cell.parameters, answer)
+
+
 def test_cell_vectors():
     """Test the simple Cell objects handling of vectors."""
     cell1 = Cell(2.456, 2.456, 6.696, 90.0, 90.0, 120.0)
